# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test parent community."""

import pytest
from invenio_access.permissions import system_identity

from invenio_communities.communities.records.api import Community


def clean_child_comm(comm, db):
    """Remove parent from community."""
    comm.parent = None
    comm.commit()
    db.session.commit()


def test_set_parent_community_with_comm_obj(parent_community, child_community, db):
    child_community.parent = parent_community

    assert child_community.parent == parent_community

    child_community.commit()
    db.session.commit()

    comm = Community.get_record(str(child_community.id))
    assert comm.parent == parent_community

    clean_child_comm(comm, db)


def test_set_parent_community_with_comm_uuid(parent_community, child_community, db):
    child_community.parent = parent_community.id

    assert child_community.parent == parent_community

    child_community.commit()
    db.session.commit()

    comm = Community.get_record(str(child_community.id))
    assert comm.parent == parent_community

    clean_child_comm(comm, db)


def test_set_parent_community_with_comm_uuid_string(
    parent_community, child_community, db
):
    child_community.parent = str(parent_community.id)

    assert child_community.parent == parent_community

    child_community.commit()
    db.session.commit()
    comm = Community.get_record(str(child_community.id))
    assert comm.parent == parent_community

    clean_child_comm(comm, db)


def test_invalid_set_parent_community_with_comm_slug(parent_community, child_community):
    with pytest.raises(ValueError, match="Invalid parent community."):
        child_community.parent = parent_community.slug


def test_invalid_set_parent_community_with_random_object(child_community):
    with pytest.raises(ValueError, match="Invalid parent community."):
        child_community.parent = dict(test="test")


def test_parent_community_dereferencing(
    community_service, parent_community, child_community, db
):
    child_community.parent = parent_community

    assert child_community.parent == parent_community

    child_community.commit()
    db.session.commit()

    comm = Community.get_record(str(child_community.id))

    community_data = community_service.schema.dump(
        comm, context=dict(identity=system_identity)
    )
    parent_community_data = community_service.schema.dump(
        parent_community, context=dict(identity=system_identity)
    )
    assert comm.parent == parent_community

    # Make sure tha the parent community is dereferenced in the child community
    assert community_data["parent"] == parent_community_data

    clean_child_comm(comm, db)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Resources serializers tests."""

from flask import g

from invenio_communities.communities.resources.serializer import (
    UICommunityJSONSerializer,
)


def test_ui_serializer(app, community, users, any_user):
    owner = users["owner"]
    reader = users["reader"]
    closed_review_comm = community.to_dict()
    closed_review_comm["access"]["review_policy"] = "closed"
    closed_review_expected_data = {
        "permissions": {"can_include_directly": False, "can_update": True}
    }

    # set current user to owner
    g.identity = owner.identity

    serialized_record = UICommunityJSONSerializer().dump_obj(closed_review_comm)
    assert (
        serialized_record["ui"]["permissions"]
        == closed_review_expected_data["permissions"]
    )

    open_review_comm = community.to_dict()
    open_review_comm["access"]["review_policy"] = "open"
    open_review_expected_data = {
        "permissions": {"can_include_directly": True, "can_update": True}
    }

    serialized_record = UICommunityJSONSerializer().dump_obj(open_review_comm)
    assert (
        serialized_record["ui"]["permissions"]
        == open_review_expected_data["permissions"]
    )

    # set user to community reader
    g.identity = reader.identity
    require_review_expected_data = {
        "permissions": {"can_include_directly": False, "can_update": False}
    }

    serialized_record = UICommunityJSONSerializer().dump_obj(open_review_comm)
    assert (
        serialized_record["ui"]["permissions"]
        == require_review_expected_data["permissions"]
    )

    # set user to any user
    g.identity = any_user.identity
    serialized_record = UICommunityJSONSerializer().dump_obj(open_review_comm)
    assert (
        serialized_record["ui"]["permissions"]
        == require_review_expected_data["permissions"]
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
# Copyright (C) 2023 KTH Royal Institute of Technology.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test community views."""

from flask import g
from invenio_records_permissions.generators import SystemProcess

from invenio_communities.permissions import CommunityPermissionPolicy
from invenio_communities.views.communities import _filter_roles
from invenio_communities.views.ui import _show_create_community_link


def _test_filter_roles(app, members, action, member_types, community_id):
    """Test _filter_roles func."""
    for role in app.config["COMMUNITIES_ROLES"]:
        role_name = role["name"]
        member = members[role_name]

        roles = _filter_roles(
            action, member_types, community_id, identity=member.identity
        )
        filtered = {role["name"] for role in roles}
        assert filtered == set(role.get("can_manage_roles", []))


def test_filter_roles_when_updating_roles(
    app, db, community_service, community, members
):
    """Test selectable roles when updating existing members' roles."""
    _test_filter_roles(
        app,
        members,
        "members_update",
        {"user", "group"},
        community.id,
    )


def test_filter_roles_when_inviting_members(
    app, db, community_service, community, members
):
    """Test selectable roles when inviting new members."""
    _test_filter_roles(
        app,
        members,
        "members_invite",
        {"user"},
        community.id,
    )


def test_filter_roles_when_adding_groups(
    app, db, community_service, community, members
):
    """Test selectable roles when adding new groups."""
    _test_filter_roles(
        app,
        members,
        "members_add",
        {"group"},
        community.id,
    )


def test_show_create_community_link(app, users, superuser_identity):
    """Test the _can_create_community function under different config settings."""
    test_users = ["reader", "curator", "manager", "owner"]
    ALWAYS_SHOW_CREATE_LINK = "COMMUNITIES_ALWAYS_SHOW_CREATE_LINK"

    # Test default config allows community creation
    assert app.config.get(ALWAYS_SHOW_CREATE_LINK) == False
    assert _show_create_community_link() == True

    # Test with config set to True
    app.config[ALWAYS_SHOW_CREATE_LINK] = True
    assert app.config.get(ALWAYS_SHOW_CREATE_LINK) == True
    assert _show_create_community_link() == True

    # Test with different user identities
    for user in test_users:
        g.identity = users[user].identity
        assert _show_create_community_link() == True

    # Test with community creation disabled
    CommunityPermissionPolicy.can_create = [SystemProcess()]
    for user in test_users:
        g.identity = users[user].identity
        assert _show_create_community_link() == False

    # Test superuser
    g.identity = superuser_identity
    assert _show_create_community_link() == True



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2024 CERN.
# Copyright (C) 2022 Northwestern University.
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community resources tests."""

import copy
from io import BytesIO
from uuid import uuid4

from invenio_communities.communities.records.api import Community
from invenio_communities.members import Member


def _assert_single_item_response(response):
    """Assert the fields present on a single item response."""
    response_fields = response.json.keys()
    fields_to_check = [
        "created",
        "id",
        "links",
        "metadata",
        "slug",
        "updated",
    ]

    for field in fields_to_check:
        assert field in response_fields


def _assert_optional_medatada_items_response(response):
    """Assert the fields present on the metadata"""
    metadata_fields = response.json["metadata"].keys()
    fields_to_check = [
        "description",
        "curation_policy",
        "page",
        "website",
        "organizations",
    ]

    for field in fields_to_check:
        assert field in metadata_fields


def _assert_optional_access_items_response(response):
    """Assert the fields present on the metadata"""
    access_fields = response.json["access"].keys()
    fields_to_check = [
        "visibility",
        "member_policy",
        "record_policy",
    ]

    for field in fields_to_check:
        assert field in access_fields


def _assert_single_item_search(response):
    """Assert the fields present on the search response"""
    response_fields = response.json.keys()
    fields_to_check = ["aggregations", "hits", "links", "sortBy"]

    for field in fields_to_check:
        assert field in response_fields


def _assert_error_fields_response(expected, response):
    """Assert the fields present in response error list and fields tested"""
    error_fields = [item["field"] for item in response.json["errors"]]

    for field in expected:
        assert field in error_fields


def _assert_error_messages_response(expected, response):
    """Assert the error messages present in response error list and expected error messages"""
    error_messages_list = [
        "Must be one of: organization, event, topic, project.",
        "Must be one of: public, private.",
        "Must be one of: public, restricted.",
        "Not empty string and less than 2000 characters allowed.",
        "Not empty string and less than 250 characters allowed.",
        "Not empty string and less than 100 characters allowed.",
    ]
    error_messages = set([item["messages"][0] for item in response.json["errors"]])

    assert expected == len(set(error_messages_list).intersection(error_messages))


def test_simple_flow(
    client, location, minimal_community, headers, owner, db, search_clear
):
    """Test a simple REST API flow."""
    client = owner.login(client)

    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)

    created_community = res.json
    id_ = created_community["id"]
    slug = created_community["slug"]

    # Read the community
    res = client.get(f"/communities/{id_}", headers=headers)
    assert res.status_code == 200
    assert res.json["metadata"] == created_community["metadata"]

    read_community = res.json
    Community.index.refresh()

    # Read the community via slug
    res = client.get(f"/communities/{slug}", headers=headers)
    assert res.status_code == 200
    assert res.json["id"] == id_

    # Search for created community
    res = client.get(f"/communities", query_string={"q": f"id:{id_}"}, headers=headers)
    assert res.status_code == 200
    assert res.json["hits"]["total"] == 1
    assert res.json["hits"]["hits"][0]["metadata"] == created_community["metadata"]

    # Update community
    data = copy.deepcopy(read_community)
    data["metadata"]["title"] = "New title"
    res = client.put(f"/communities/{id_}", headers=headers, json=data)
    assert res.status_code == 200
    assert res.json["metadata"]["title"] == "New title"

    updated_community = res.json
    Community.index.refresh()

    # Search for updated commmunity
    res = client.get(f"/communities", query_string={"q": f"id:{id_}"}, headers=headers)
    assert res.status_code == 200
    assert res.json["hits"]["total"] == 1
    assert res.json["hits"]["hits"][0]["metadata"] == updated_community["metadata"]
    data = res.json["hits"]["hits"][0]
    assert updated_community["metadata"]["title"] == "New title"

    # Delete community
    res = client.delete(f"/communities/{id_}", headers=headers)
    assert res.status_code == 204

    # Read again community, should return 404
    res = client.get(f"/communities/{id_}", headers=headers)
    assert res.status_code == 410
    assert res.json["message"] == "The record has been deleted."


def test_post_schema_validation(
    client, location, minimal_community, headers, owner, search_clear, db
):
    """Test the validity of community json schema"""
    client = owner.login(client)

    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    assert minimal_community["metadata"] == created_community["metadata"]

    # Assert required fields
    assert created_community["metadata"] == {"title": "My Community"}
    assert created_community["access"]["visibility"] == "public"

    # Assert required enums
    data = copy.deepcopy(minimal_community)
    data["metadata"]["type"] = {"id": "foobar", "title": "foobar"}
    res = client.post("/communities", headers=headers, json=data)
    assert res.status_code == 400

    data = copy.deepcopy(minimal_community)
    data["access"]["visibility"] = "foobar"
    res = client.post("/communities", headers=headers, json=data)
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    _assert_error_fields_response(set(["access.visibility"]), res)
    _assert_error_messages_response(1, res)

    # Delete the community
    res = client.delete(f'/communities/{created_community["id"]}', headers=headers)
    assert res.status_code == 204


def test_post_metadata_schema_validation(
    client, location, minimal_community, headers, db, owner, search_clear
):
    """Test the validity of community metadata schema"""
    client = owner.login(client)

    # Alter paypload for each field for test
    data = copy.deepcopy(minimal_community)

    # Assert field size constraints  (id, title, description, curation policy, page)
    # ID max 100
    data["slug"] = "x" * 101
    res = client.post("/communities", headers=headers, json=data)
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]["field"] == "slug"
    # assert res.json["errors"][0]['messages'] == ['Not a valid string.']

    # Title max 250
    data["slug"] = "my_comm"
    data["metadata"]["title"] = "x" * 251
    res = client.post("/communities", headers=headers, json=data)
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]["field"] == "metadata.title"
    # assert res.json["errors"][0]['messages'] == ['Title is too long.']

    # Description max 250
    data["metadata"]["title"] = "New Title"
    data["metadata"]["description"] = "x" * 251
    res = client.post("/communities", headers=headers, json=data)
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]["field"] == "metadata.description"
    # assert res.json["errors"][0]['messages'] == ['Description is too long.']

    # Curation policy max 50000
    data["metadata"]["description"] = "basic description"
    data["metadata"]["curation_policy"] = "x" * 50001
    res = client.post("/communities", headers=headers, json=data)
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]["field"] == "metadata.curation_policy"
    # assert res.json["errors"][0]['messages'] == ['Curation policy is too long.']

    # Curation policy max 50000
    data["metadata"]["curation_policy"] = "no policy"
    data["metadata"]["page"] = "".join([str(i) for i in range(50001)])
    res = client.post("/communities", headers=headers, json=data)
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]["field"] == "metadata.page"
    # assert res.json["errors"][0]['messages'] == ['Page is too long.']

    data["metadata"]["page"] = "basic page"
    res = client.post("/communities", headers=headers, json=data)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)

    # Delete the community
    res = client.delete(f'/communities/{data["slug"]}', headers=headers)
    assert res.status_code == 204


def test_post_community_with_existing_id(
    client, location, minimal_community, headers, db, search_clear, owner
):
    """Test create two communities with the same id"""
    client = owner.login(client)

    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community["id"]

    # Creta another community with the same id
    minimal_community["id"] = id_
    res = client.post("/communities", headers=headers, json=minimal_community)
    assert res.status_code == 400
    assert (
        res.json["errors"][0]["messages"][0]
        == "A community with this identifier already exists."
    )

    # Delete the community
    res = client.delete(f"/communities/{id_}", headers=headers)
    assert res.status_code == 204


def test_post_community_with_deleted_id(
    client, location, minimal_community, headers, db, search_clear, owner
):
    """Test create a community with a deleted id"""
    client = owner.login(client)

    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community["id"]

    # Delete community
    res = client.delete(f"/communities/{id_}", headers=headers)
    assert res.status_code == 204

    # Create another community with the same id
    minimal_community["id"] = id_
    res = client.post("/communities", headers=headers, json=minimal_community)
    assert res.status_code == 400
    assert (
        res.json["errors"][0]["messages"][0]
        == "A community with this identifier already exists."
    )


def test_post_self_links(
    client, location, minimal_community, headers, db, search_clear, owner
):
    """Test self links generated after post"""
    client = owner.login(client)

    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community["id"]
    slug = created_community["slug"]
    # assert '/'.join(created_community['links']['self'].split('/')[-2:]) == f'communities/{id_}'
    assert (
        created_community["links"]["self"]
        == f"https://127.0.0.1:5000/api/communities/{id_}"
    )
    assert (
        created_community["links"]["self_html"]
        == f"https://127.0.0.1:5000/communities/{slug}"
    )

    # Delete the community
    res = client.delete(f"/communities/{id_}", headers=headers)
    assert res.status_code == 204


def test_simple_search_response(
    client,
    location,
    minimal_community,
    owner,
    headers,
    search_clear,
    fake_communities,
    community_types,
):
    """Test get/list and search functionality"""
    client = owner.login(client)

    community_copy = copy.deepcopy(minimal_community)
    community_copy["metadata"]["type"] = community_types[-1]

    # Create many communities,
    slug_oldest, slug_newest, num_each, total = fake_communities

    # Search for any commmunity, default order newest
    res = client.get(f"/communities", query_string={"q": f""}, headers=headers)
    assert res.status_code == 200
    _assert_single_item_search(res)
    assert res.json["hits"]["total"] == total
    assert res.json["hits"]["hits"][0]["slug"] == slug_newest
    assert res.json["hits"]["hits"][0]["metadata"] == community_copy["metadata"]

    # Search filter for the second commmunity
    res = client.get(
        f"/communities", query_string={"q": "metadata.type.id:project"}, headers=headers
    )
    assert res.status_code == 200
    _assert_single_item_search(res)
    assert res.json["hits"]["total"] == num_each

    # Sort results by oldest
    res = client.get(
        f"/communities", query_string={"q": "", "sort": "oldest"}, headers=headers
    )
    assert res.status_code == 200
    assert res.json["hits"]["hits"][0]["slug"] == slug_oldest

    # Test for page and size
    res = client.get(
        f"/communities",
        query_string={"q": "", "size": "5", "page": "2"},
        headers=headers,
    )
    assert res.status_code == 200
    assert res.json["hits"]["total"] == total
    assert len(res.json["hits"]["hits"]) == 5


def test_simple_get_response(
    client,
    location,
    full_community,
    headers,
    db,
    search_clear,
    owner,
    community_type_record,
):
    """Test get response json schema"""
    client = owner.login(client)
    community_copy = copy.deepcopy(full_community)
    # Dereferenced community has attribute 'title' in 'type'
    community_copy["metadata"]["type"].update({"title": {"en": "Event"}})

    # Create a community
    res = client.post("/communities", headers=headers, json=full_community)
    Member.index.refresh()
    assert res.status_code == 201
    _assert_single_item_response(res)
    _assert_optional_medatada_items_response(res)

    created_community = res.json

    # assert full_community['access'] == created_community['access']

    assert community_copy["metadata"] == created_community["metadata"]
    id_ = created_community["id"]

    # Read the community
    res = client.get(f"/communities/{id_}", headers=headers)
    assert res.status_code == 200
    _assert_single_item_response(res)
    _assert_optional_medatada_items_response(res)
    # _assert_optional_access_items_response(res)
    # assert full_community['access'] == res.json['access']
    assert community_copy["metadata"] == res.json["metadata"]

    # Read a non-existed community
    res = client.get(f"/communities/{id_[:-1]}", headers=headers)
    assert res.status_code == 404
    # assert res.message == 'The persistent identifier does not exist.'


def test_simple_put_response(
    client,
    location,
    minimal_community,
    headers,
    db,
    search_clear,
    owner,
    community_type_record,
):
    """Test put response basic functionality"""
    client = owner.login(client)
    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community["id"]
    Member.index.refresh()

    data = copy.deepcopy(minimal_community)
    data["metadata"] = {
        "title": "New Community",
        "description": "This is an example Community.",
        "type": {
            "id": "event",
        },
        "curation_policy": "This is the kind of records we accept.",
        "page": "Information for my community.",
        "website": "https://inveniosoftware.org/",
    }

    data["access"] = {
        "visibility": "restricted",
        "members_visibility": "restricted",
        "member_policy": "closed",
        "record_policy": "closed",
    }

    res = client.put(f"/communities/{id_}", headers=headers, json=data)
    # Dereferenced community has attribute 'title' in 'type'
    data["metadata"]["type"].update({"title": {"en": "Event"}})
    assert res.status_code == 200
    assert res.json["id"] == id_
    assert res.json["metadata"] == data["metadata"]
    # assert access_== data["access"]
    assert res.json["revision_id"] == int(created_community["revision_id"]) + 1

    # Update non-existing community
    res = client.put(f"/communities/{id_[:-1]}", headers=headers, json=data)
    assert res.status_code == 404
    assert res.json["message"] == "The persistent identifier does not exist."

    # Update deleted community
    res = client.delete(f"/communities/{id_}", headers=headers)
    assert res.status_code == 204
    data["metadata"]["title"] = "Deleted community"
    res = client.put(f"/communities/{id_}", headers=headers, json=data)
    assert res.status_code == 410
    assert res.json["message"] == "The record has been deleted."


def test_update_renamed_record(
    client, location, minimal_community, headers, db, search_clear, owner
):
    """Test to update renamed entity."""
    client = owner.login(client)
    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community["id"]
    slug = created_community["slug"]

    # Rename the community
    data = copy.deepcopy(minimal_community)
    data["slug"] = "renamed_comm"
    res = client.post(f"/communities/{id_}/rename", headers=headers, json=data)
    assert res.status_code == 200
    renamed_community = res.json
    renamed_id_ = renamed_community["id"]
    data["access"] = {
        "visibility": "restricted",
        "members_visibility": "restricted",
        "member_policy": "closed",
        "record_policy": "closed",
    }
    res = client.put(f"/communities/{renamed_id_}", headers=headers, json=data)
    assert res.status_code == 200
    assert res.json["id"] == renamed_id_
    assert res.json["metadata"] == data["metadata"]
    assert res.json["revision_id"] == int(renamed_community["revision_id"]) + 1


def test_simple_delete_response(
    client, location, minimal_community, headers, db, search_clear, owner
):
    """Test delete and request deleted community."""
    client = owner.login(client)

    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community["id"]

    # Delete community
    res = client.delete(f"/communities/{id_}", headers=headers)
    assert res.status_code == 204

    # Read the community
    res = client.get(f"/communities/{id_}", headers=headers)
    assert res.status_code == 410
    assert res.json["message"] == "The record has been deleted."

    # Delete non-existing community
    res = client.delete(f"/communities/{id_[:-1]}", headers=headers)
    assert res.status_code == 404
    assert res.json["message"] == "The persistent identifier does not exist."


def test_logo_flow(
    app, client, location, minimal_community, headers, db, search_clear, owner
):
    """Test logo workflow."""
    client = owner.login(client)

    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    assert res.status_code == 201
    created_community = res.json
    id_ = created_community["id"]
    assert (
        created_community["links"]["logo"]
        == f"https://127.0.0.1:5000/api/communities/{id_}/logo"
    )
    Member.index.refresh()

    # Get non-existent logo
    res = client.get(f"/communities/{id_}/logo")
    assert res.status_code == 404
    assert res.json["message"] == "No logo exists for this community."

    # Delete non-existent logo
    res = client.delete(f"/communities/{id_}/logo", headers=headers)
    assert res.status_code == 404
    assert res.json["message"] == "No logo exists for this community."

    # Update logo
    res = client.put(
        f"/communities/{id_}/logo",
        headers={
            **headers,
            "content-type": "application/octet-stream",
        },
        data=BytesIO(b"logo"),
    )
    assert res.status_code == 200
    assert res.json["size"] == 4

    # Get logo
    res = client.get(f"/communities/{id_}/logo")
    assert res.status_code == 200
    assert res.data == b"logo"

    # Update logo again
    res = client.put(
        f"/communities/{id_}/logo",
        headers={
            **headers,
            "content-type": "application/octet-stream",
        },
        data=BytesIO(b"new_logo"),
    )
    assert res.status_code == 200
    assert res.json["size"] == 8

    # Get new logo
    res = client.get(f"/communities/{id_}/logo")
    assert res.status_code == 200
    assert res.data == b"new_logo"

    # Delete logo with unauthorized user
    with app.test_client() as anon_client:
        res = anon_client.delete(f"/communities/{id_}/logo", headers=headers)
        assert res.status_code == 403

    # Delete logo
    res = client.delete(f"/communities/{id_}/logo", headers=headers)
    assert res.status_code == 204

    # Try to get deleted logo
    res = client.get(f"/communities/{id_}/logo")
    assert res.status_code == 404
    assert res.json["message"] == "No logo exists for this community."

    # TODO: Delete community and try all of the above operations


def test_logo_max_content_length(
    app, client, location, minimal_community, headers, db, search_clear, owner
):
    """Test logo max size."""
    client = owner.login(client)

    # Create a community
    res = client.post("/communities", headers=headers, json=minimal_community)
    assert res.status_code == 201
    created_community = res.json
    id_ = created_community["id"]
    assert (
        created_community["links"]["logo"]
        == f"https://127.0.0.1:5000/api/communities/{id_}/logo"
    )
    Member.index.refresh()

    # Update app max size for community logos
    max_size = 10**6
    app.config["COMMUNITIES_LOGO_MAX_FILE_SIZE"] = max_size

    # Update logo with big content
    logo_data = b"logo" * (max_size + 1)
    res = client.put(
        f"/communities/{id_}/logo",
        headers={
            **headers,
            "content-type": "application/octet-stream",
        },
        data=BytesIO(logo_data),
    )
    assert res.status_code == 400

    # Update logo with small content
    logo_data = b"logo"
    res = client.put(
        f"/communities/{id_}/logo",
        headers={
            **headers,
            "content-type": "application/octet-stream",
        },
        data=BytesIO(logo_data),
    )
    assert res.status_code == 200


def test_invalid_community_ids_create(
    client, location, minimal_community, headers, db, search_clear, owner
):
    client = owner.login(client)
    # Create a community with invalid ID
    minimal_community["slug"] = "comm id"
    res = client.post("/communities", headers=headers, json=minimal_community)
    assert res.status_code == 400
    assert (
        res.json["errors"][0]["messages"][0]
        == "The ID should contain only letters with numbers or dashes."
    )


def test_invalid_community_ids(
    client, location, minimal_community, headers, db, search_clear, owner
):
    """Test for invalid community IDs handling."""
    client = owner.login(client)

    minimal_community["slug"] = "comm_id"
    res = client.post("/communities", headers=headers, json=minimal_community)
    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community["id"]
    Member.index.refresh()

    # Rename the community
    data = copy.deepcopy(minimal_community)
    data["slug"] = "Renamed comm"
    res = client.post(f"/communities/{id_}/rename", headers=headers, json=data)
    assert res.status_code == 400
    assert (
        res.json["errors"][0]["messages"][0]
        == "The ID should contain only letters with numbers or dashes."
    )

    data = copy.deepcopy(minimal_community)
    data = {}
    res = client.post(f"/communities/{id_}/rename", headers=headers, json=data)
    assert res.status_code == 400
    assert res.json["errors"][0]["messages"][0] == "Missing data for required field."

    new_id = "Renamed_comm"
    data["slug"] = new_id
    res = client.post(f"/communities/{id_}/rename", headers=headers, json=data)
    assert res.status_code == 200
    assert res.json["slug"] == new_id.lower()

    # UUID is invalid
    minimal_community["slug"] = str(uuid4())
    res = client.post("/communities", headers=headers, json=minimal_community)
    assert res.status_code == 400


def test_featured_communities(
    client, location, minimal_community, headers, db, search_clear, admin
):
    """Test featured communities endpoints."""
    client = admin.login(client)

    minimal_community["slug"] = "comm_id"
    res = client.post("/communities", headers=headers, json=minimal_community)
    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    c_id_ = created_community["id"]
    Member.index.refresh()

    # Fetch featured communities
    res = client.get("/communities/featured", headers=headers, json=minimal_community)
    assert res.status_code == 200
    assert res.json["hits"]["total"] == 0
    _assert_single_item_search(res)

    # Other endpoints are currently only accessible with superuser-access
    # At least we know if the endpoints are accessible and params are provided
    res = client.get(f"/communities/{c_id_}/featured", headers=headers)
    assert res.status_code == 403

    res = client.post(f"/communities/{c_id_}/featured", headers=headers, json={})
    assert res.status_code == 403

    # featured entry does not exist
    res = client.put(f"/communities/{c_id_}/featured/0", headers=headers, json={})
    assert res.status_code == 404

    res = client.delete(f"/communities/{c_id_}/featured/0", headers=headers)
    assert res.status_code == 404


def test_simple_flow_restricted_community(
    app,
    client,
    location,
    minimal_restricted_community_1,
    minimal_restricted_community_2,
    headers,
    owner,
    db,
):
    """Test a simple REST API flow."""
    client = owner.login(client)

    # Create a community
    res = client.post(
        "/communities", headers=headers, json=minimal_restricted_community_1
    )
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community["id"]
    slug = created_community["slug"]

    # Read the community
    res = client.get(f"/communities/{id_}", headers=headers)
    assert res.status_code == 200
    assert res.json["metadata"] == created_community["metadata"]

    read_community = res.json
    Community.index.refresh()

    app.config["COMMUNITIES_ALLOW_RESTRICTED"] = False

    # Create a new community, which should default to public
    res = client.post(
        "/communities", headers=headers, json=minimal_restricted_community_2
    )
    Member.index.refresh()

    assert res.status_code == 403


def test_permissions_modify_community_visibility(
    app,
    client,
    location,
    minimal_community,
    headers,
    owner,
    db,
):
    """Test modifying community visibility."""

    app.config["COMMUNITIES_ALLOW_RESTRICTED"] = False

    client = owner.login(client)
    # Create a public community
    res = client.post("/communities", headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)

    created_community = res.json
    id_ = created_community["id"]
    slug = created_community["slug"]

    # Read the community
    res = client.get(f"/communities/{id_}", headers=headers)
    assert res.status_code == 200
    assert res.json["metadata"] == created_community["metadata"]

    read_community = res.json
    Community.index.refresh()

    # try to change public community to restricted
    data = copy.deepcopy(read_community)
    data["metadata"]["title"] = "New title"
    data["access"]["visibility"] = "restricted"
    res = client.put(f"/communities/{id_}", headers=headers, json=data)
    assert res.status_code == 403

    # try to change other attributes but leave the visibility the same
    # happens on community settings page
    data["metadata"]["title"] = "New title 2"
    data["access"]["visibility"] = "public"
    res = client.put(f"/communities/{id_}", headers=headers, json=data)
    assert res.status_code == 200


def test_permissions_modify_community_to_public(
    app,
    client,
    location,
    minimal_restricted_community_1,
    headers,
    owner,
    db,
):
    """Test modifying community visibility."""
    app.config["COMMUNITIES_ALLOW_RESTRICTED"] = True
    client = owner.login(client)
    # Create a public community
    res = client.post(
        "/communities", headers=headers, json=minimal_restricted_community_1
    )
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)

    created_community = res.json
    id_ = created_community["id"]
    slug = created_community["slug"]

    # Read the community
    res = client.get(f"/communities/{id_}", headers=headers)
    assert res.status_code == 200
    assert res.json["metadata"] == created_community["metadata"]

    read_community = res.json
    Community.index.refresh()
    app.config["COMMUNITIES_ALLOW_RESTRICTED"] = False

    # try to change public community to restricted
    data = copy.deepcopy(read_community)
    data["metadata"]["title"] = "New title"
    data["access"]["visibility"] = "public"
    res = client.put(f"/communities/{id_}", headers=headers, json=data)
    assert res.status_code == 403



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2024 CERN.
# Copyright (C) 2022 Northwestern University.
# Copyright (C) 2022-2023 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Components."""

from flask import current_app
from invenio_access.permissions import system_identity, system_process
from invenio_db import db
from invenio_i18n import lazy_gettext as _
from invenio_oaiserver.models import OAISet
from invenio_pidstore.errors import PIDDeletedError, PIDDoesNotExistError
from invenio_records_resources.services.records.components import (
    MetadataComponent,
    RelationsComponent,
    ServiceComponent,
)
from marshmallow.exceptions import ValidationError

from ...proxies import current_roles
from ...utils import on_user_membership_change
from ..records.systemfields.access import VisibilityEnum
from ..records.systemfields.deletion_status import CommunityDeletionStatusEnum


class PIDComponent(ServiceComponent):
    """Service component for Community PIDs."""

    def set_slug(self, record, slug):
        """Set slug."""
        error = ValidationError(
            _("A community with this identifier already exists."), field_name="slug"
        )
        try:
            self.service.record_cls.pid.resolve(slug)
            raise error
        except PIDDeletedError:
            raise error
        except PIDDoesNotExistError:
            record.slug = slug

    def create(self, identity, record=None, data=None, **kwargs):
        """Create a Community PID from its metadata."""
        self.set_slug(record, data["slug"])

    def update(self, identity, record=None, data=None, **kwargs):
        """Rename the Community PIDs value."""
        if "slug" in data and record.slug != data["slug"]:
            raise ValidationError(
                _("The ID should be modified through the renaming URL instead."),
                field_name="slug",
            )

    def rename(self, identity, record=None, data=None, **kwargs):
        """Rename the Community PIDs value."""
        if record.slug == data["slug"]:
            raise ValidationError(
                _("A new ID value is required for the renaming."),
                field_name="slug",
            )
        self.set_slug(record, data["slug"])


class CommunityAccessComponent(ServiceComponent):
    """Service component for access integration."""

    def _populate_access_and_validate(self, identity, data, record, **kwargs):
        """Populate and validate the community's access field."""
        is_modifying_access = record is not None and "access" in data

        if not is_modifying_access:
            return

        access = data.get("access", {})
        new_visibility = access.get("visibility")

        record_has_defined_access = "access" in record
        is_restricting = new_visibility == "restricted"

        if (
            VisibilityEnum(record.access.visibility) != VisibilityEnum(new_visibility)
            and record_has_defined_access
        ):
            self.service.require_permission(identity, "manage_access", record=record)
        if not record_has_defined_access and is_restricting:
            self.service.require_permission(
                identity, "create_restricted", record=record
            )

        record.setdefault("access", {})
        record["access"].update(access)
        record.access.refresh_from_dict(record.get("access"))

    def create(self, identity, data=None, record=None, **kwargs):
        """Add basic ownership fields to the record."""
        self._populate_access_and_validate(identity, data, record, **kwargs)

    def update(self, identity, data=None, record=None, **kwargs):
        """Update handler."""
        self._populate_access_and_validate(identity, data, record, **kwargs)


class OwnershipComponent(ServiceComponent):
    """Service component for owner membership integration."""

    def create(self, identity, data=None, record=None, **kwargs):
        """Make an owner member from the identity."""
        if system_process in identity.provides:
            return

        member = {
            "type": "user",
            "id": str(identity.id),
        }
        self.service.members.add(
            # the user is not yet owner of the community (is being added)
            # therefore we cannot use `identity`
            system_identity,
            record.id,
            {"members": [member], "role": current_roles.owner_role.name},
            uow=self.uow,
        )

        # Invalidate the membership cache
        on_user_membership_change(identity=identity)


class FeaturedCommunityComponent(ServiceComponent):
    """Service component for featured community integration."""

    def featured_create(self, identity, data=None, record=None, **kwargs):
        """Featured create."""
        if record.access.visibility != "public":
            raise ValidationError(
                _("The community is not public."), field_name="community_id"
            )


class OAISetComponent(ServiceComponent):
    """Service component for OAI set integration."""

    def _retrieve_set(self, slug):
        return OAISet.query.filter(OAISet.spec == self._create_set_spec(slug)).first()

    def _create_set_spec(self, community_slug):
        oai_sets_prefix = current_app.config["COMMUNITIES_OAI_SETS_PREFIX"]
        return "{prefix}{slug}".format(prefix=oai_sets_prefix, slug=community_slug)

    def _create_set_description(self, community_title):
        # NOTE: Does not require translation since this description can also be changed by an admin
        return "Records belonging to the community '{title}'".format(
            title=community_title
        )

    def _create_set_from_community(self, record):
        community_slug = record.slug
        community_title = record.metadata["title"]

        community_set = OAISet()
        community_set.name = community_title
        community_set.system_created = True
        community_set.description = self._create_set_description(community_title)
        community_set.spec = self._create_set_spec(community_slug)
        community_set.search_pattern = "parent.communities.ids:{id}".format(
            id=record.id
        )

        return community_set

    def create(self, identity, data=None, record=None, **kwargs):
        """Create a set for the community."""
        if record.access.visibility != "public":
            return

        community_set = self._create_set_from_community(record)
        # NOTE: will be indexed via a listener in oaiserver module
        db.session.add(community_set)

    def delete(self, identity, data=None, record=None, **kwargs):
        """Remove the set for the community."""
        community_set = self._retrieve_set(kwargs.get("slug") or record.slug)
        if not community_set:
            return

        # NOTE: will be removed from index via listener in oaiserver module
        db.session.delete(community_set)

    def update(self, identity, data=None, record=None, **kwargs):
        """Update set accordingly."""
        if record.access.visibility != "public":
            # delete set if the community is not public anymore
            self.delete(identity, data, record, **kwargs)
            return

        community_set = self._retrieve_set(record.slug)
        if not community_set:
            # create new set because community is now public
            self.create(identity, data, record, **kwargs)
            return

        # just update set with new name and description
        community_set.name = record.metadata["title"]
        community_set.description = self._create_set_description(community_set.name)
        db.session.add(community_set)

    def rename(self, identity, record=None, data=None, old_slug=None, **kwargs):
        """Delete old set and create new set on ID change."""
        if record.access.visibility != "public":
            return

        # Since we want to delete the set created with the old set, we can not
        # use the already updated pid_value
        self.delete(identity, data=data, record=record, **kwargs, slug=old_slug)
        community_set = self._create_set_from_community(record)
        db.session.add(community_set)


class CustomFieldsComponent(ServiceComponent):
    """Service component for custom fields."""

    def create(self, identity, data=None, record=None, errors=None, **kwargs):
        """Inject parsed custom fields to the record."""
        record.custom_fields = data.get("custom_fields", {})

    def update(self, identity, data=None, record=None, **kwargs):
        """Inject parsed custom fields to the record."""
        record.custom_fields = data.get("custom_fields", {})


class CommunityDeletionComponent(ServiceComponent):
    """Service component for record deletion."""

    def delete(self, identity, data=None, record=None, **kwargs):
        """Set the community's deletion status and tombstone information."""
        # Set the record's deletion status and tombstone information
        record.deletion_status = CommunityDeletionStatusEnum.DELETED
        record.tombstone = data

        # Set `removed_by` information for the tombstone
        record.tombstone.removed_by = identity.id

    def update_tombstone(self, identity, data=None, record=None, **kwargs):
        """Update the community's tombstone information."""
        record.tombstone = data

    def restore(self, identity, data=None, record=None, **kwargs):
        """Reset the community's deletion status and tombstone information."""
        record.deletion_status = CommunityDeletionStatusEnum.PUBLISHED

        # Remove the tombstone information
        record.tombstone = None

    def mark(self, identity, data=None, record=None, **kwargs):
        """Mark the community for purge."""
        record.deletion_status = CommunityDeletionStatusEnum.MARKED
        record.tombstone = record.tombstone

    def unmark(self, identity, data=None, record=None, **kwargs):
        """Unmark the community for purge, resetting it to soft-deleted state."""
        record.deletion_status = CommunityDeletionStatusEnum.DELETED
        record.tombstone = record.tombstone


class CommunityThemeComponent(ServiceComponent):
    """Service component for Community theme."""

    def create(self, identity, data=None, record=None, **kwargs):
        """Inject parsed theme to the record."""
        if "theme" in data:
            self.service.require_permission(identity, "set_theme", record=record)
            record["theme"] = data["theme"]

    def update(self, identity, data=None, record=None, errors=None, **kwargs):
        """Inject parsed theme to the record."""
        stored_record_theme = record.get("theme")
        if "theme" in data:
            # if theme set to None then it is a delete operation
            if data["theme"] is None:
                if stored_record_theme is not None:
                    self.service.require_permission(
                        identity, "delete_theme", record=record
                    )
                    # delete theme from record and data
                    record.pop("theme", None)
                # We always pop the {theme: None} from the data so we don't store None
                # value in the record
                data.pop("theme", None)
            elif data["theme"] != stored_record_theme:
                # check update permissions for theme only if incoming theme is
                # different from stored. Check is needed, so we don't apply the theme
                # permission check when other community information is updated
                self.service.require_permission(identity, "set_theme", record=record)
                record["theme"] = data["theme"]


class CommunityParentComponent(ServiceComponent):
    """Service Component for Community parent."""

    def _validate_and_get_parent(self, parent_data, child):
        """Validate and return parent community."""
        if not parent_data:
            return None
        try:
            parent = self.service.record_cls.pid.resolve(parent_data["id"])
            if not parent.children.allow:
                raise ValidationError("Assigned parent is not allowed to be a parent.")
            elif child.children.allow:
                raise ValidationError(
                    "Community allowed to be a parent can't be a child."
                )
            elif parent.parent:
                raise ValidationError(
                    "Assigned parent community cannot also have a parent."
                )
            elif child.id == parent.id:
                raise ValidationError(
                    "Assigned parent community cannot be the same as child."
                )
        except PIDDoesNotExistError:
            raise ValidationError("Assigned parent community does not exist.")
        return parent

    def create(self, identity, data=None, record=None, **kwargs):
        """Inject parsed theme to the record."""
        if "parent" in data:
            self.service.require_permission(identity, "manage_parent", record=record)
            parent = self._validate_and_get_parent(data["parent"], record)
            record.parent = parent

    def update(self, identity, data=None, record=None, **kwargs):
        """Update parent community of a community."""
        if not data:
            return
        existing_parent_id = record.parent and str(record.parent.id)
        new_parent_id = (data.get("parent") or {}).get("id")
        if "parent" in data and new_parent_id != existing_parent_id:
            self.service.require_permission(identity, "manage_parent", record=record)
            parent = self._validate_and_get_parent(data["parent"], record)
            record.parent = parent


class ChildrenComponent(ServiceComponent):
    """Service component for children integration."""

    def _populate_and_validate(self, identity, data, record, **kwargs):
        """Populate and validate the community's children field."""
        # We check if data is passed and is different to the stored or default value
        existing_value = record.children.dump()
        if "children" in data and data["children"] != existing_value:
            self.service.require_permission(identity, "manage_children", record=record)
            record.children = record.children.from_dict(data["children"])

    def create(self, identity, data=None, record=None, **kwargs):
        """Create handler."""
        self._populate_and_validate(identity, data, record, **kwargs)

    def update(self, identity, data=None, record=None, **kwargs):
        """Update handler."""
        self._populate_and_validate(identity, data, record, **kwargs)


DefaultCommunityComponents = [
    MetadataComponent,
    CommunityThemeComponent,
    CustomFieldsComponent,
    PIDComponent,
    RelationsComponent,
    CommunityAccessComponent,
    OwnershipComponent,
    FeaturedCommunityComponent,
    OAISetComponent,
    CommunityDeletionComponent,
    ChildrenComponent,
    CommunityParentComponent,
]
